function [p, prob_v_condy1, prob_vy1_max]=DDS_mode_test(y1, y2)
% Function gives the p-value of the test described by Das-Dominitz and Soest (1999) for the expectations of  discrete outcomes
% This function makes the test that agents are both rational and express expectations as their subjective Mode

% y1 is a [N x 1] column vector of expectations
% y2 is a [N x 1] column vector of results 
% y1 and y2 are assumed to be discrete (i.e, values = 0, 1, 2, 3,..., n belong to integer numbers)  

% asymptotic variance for Pr( y2=k | y1 = k)-Pr( y2=j | y1 = k) is 2*p1=2*Pr( y2=k | y1 = k) 

% Results:
% p is the p-value for the test: Pr( y2=k | y1 = k)-max(j: Pr( y2=j | y1 = k) )
% prob_v_condy1 is the pdf  Pr( y2== j | y1 = k) where j is row and k is column
% [prob_vy1_max has the values of the agents' guesses ordered in the first column (v),
% 2nd column has the sample estimate of the pdf Pr( y2=k | y1 = k)
% 3rd column has the sample estimate of the pdf Pr( y2=mode(y2 | k) | y1 = k) for each value k
% 4th column has the value of the mode(y2 | k) that is closest to k (for values of j<=k)
% 5th column has the value of the mode(y2 | k) that is closest to k (for values of j>=k)

vmin = min([y1; y2]);  vmax = max([y1; y2]);  v = vmin:vmax;
obs_v = histc([y1; y2], v);
for i = 1:size(obs_v, 1);
    if obs_v(i, 1)==0;
        v(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
v = v(logical( isnan(v)==0 ) )';  % values that never appear will not be used for computing probabilities

nr_v = size(v, 1);
prob_v_condy1 = NaN.*zeros(nr_v);
p = NaN.*zeros(nr_v,1); p_all = NaN.*zeros(nr_v,nr_v-1);
prob_vy1_max = [v, NaN.*zeros(nr_v,4)];  % matrix has values of v (1st column) and prob(y2= v | y1=v) (2nd column)
        % 3rd column - max( prob(y2= j | y1=v) ), 
        % 4th column - j is closest value to v such that prob( j| y1=v) = max( prob(y2= j | y1=v) ) and j<=v
        % 5th column - j is closest value to v such that prob( j| y1=v) = max( prob(y2= j | y1=v) ) and j>=v
        
for j =1:nr_v;
    ind=find( y1 == v(j,1) & isnan(y2)==0 );   Ny=size(ind, 1); scale=sqrt(Ny); %1,sqrt(Ny),N
    
    if Ny>=2; % if size is positive, estimate conditional sample-probabilities (otherwise, they are missing)
    prob_v_condy1(:, j) = histc( y2(ind,1) , v')/Ny;    
    prob_vy1_max(j, 2) = prob_v_condy1(j,j);  prob_vy1_max(j, 3) = max( prob_v_condy1(:,j) );
    
    var_pj = 2*prob_vy1_max(j, 2); %This is Das-Dominitz-Soest expression
    if var_pj==0;  % This can happen if prob_vy1_max(j, 2) or prob_vy1_max(j, 3) are 0 (set variance to positive nr, but it doesn't matter
    var_pj = 0.25;                        % since t-statistic = 0)     
    end
      
    p(j,1) = (scale/sqrt( var_pj )) * ( prob_vy1_max(j, 2) - prob_vy1_max(j, 3) );  % Das-Dominitz-Soest test against maximum value encountered sqrt(Ny)
    j3=1;
    for j2=1:nr_v;
    if j2~=j;    
    p_all(j,j3) = (scale/sqrt( var_pj )) * ( prob_vy1_max(j, 2) - prob_v_condy1(j2, j) );  % Das-Dominitz-Soest test against maximum value encountered sqrt(Ny)
    j3=j3+1;
    end
    end
    
    ind = find( prob_v_condy1(1:j,j)==prob_vy1_max(j, 3));
    if size(ind, 1)>=1;
    prob_vy1_max(j, 4) = v(ind(size(ind, 1),1), 1); 
    end
    ind = find( prob_v_condy1(j:nr_v,j)==prob_vy1_max(j, 3));
    if size(ind, 1)>=1;
    prob_vy1_max(j, 5) = v(j-1+ind(1,1), 1); 
    end
          
    end
end

% now calculate p-value of one-sided confidence test (note that p<=0, so p-value must be <=0.5)
p = normcdf( p );
p=min(normcdf(p_all), [], 2);

