function [p]=DDS_mode_test_ALL(y1, y2)
% Function gives the p-value of the test described by Madeira (2015) for the expectations of  discrete outcomes
% This function makes the test that agents are both rational and express expectations as their subjective Mode
% It tests jointly that all the agents are rational for whatever prediction
% y1=1,...,K is reported

% y1 is a [N x 1] column vector of expectations
% y2 is a [N x 1] column vector of results 
% y1 and y2 are assumed to be discrete (i.e, values = 0, 1, 2, 3,..., n belong to integer numbers)  

% asymptotic matrix is used for Pr( y2=k | y1 = k)-Pr( y2=j | y1 = k) 
% p1*(1-p1)+p2*(1-p2)+2*p1*p2

% Results:
% p is the p-value for the test: Pr( y2=k | y1 = k)-max(j: Pr( y2=j | y1 = k) )

% prob_v_condy1 is the pdf  Pr( y2== j | y1 = k) where j is row and k is column
% [prob_vy1_max has the values of the agents' guesses ordered in the first column (v),
% 2nd column has the sample estimate of the pdf Pr( y2=k | y1 = k)
% 3rd column has the sample estimate of the pdf Pr( y2=mode(y2 | k) | y1 = k) for each value k
% 4th column has the value of the mode(y2 | k) that is closest to k (for values of j<=k)
% 5th column has the value of the mode(y2 | k) that is closest to k (for values of j>=k)

vmin=min([y1; y2]); vmax=max([y1; y2]); v=vmin:vmax; obs_v=histc([y1; y2], v);  %vmin=min([y1; y2]); vmax=max([y1; y2]); v=vmin:vmax; v=v';
for i = 1:size(obs_v, 1);
    if obs_v(i, 1)==0;
        v(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
v=v(logical( isnan(v)==0 ) )';  % values that never appear will not be used for computing probabilities

nr_v = size(v, 1); prob_v_condy1 = NaN.*zeros(nr_v); p = NaN.*zeros(nr_v,1);
prob_vy1_max = [v, NaN.*zeros(nr_v,4)];  % matrix has values of v (1st column) and prob(y2= v | y1=v) (2nd column)
        % 3rd column - max( prob(y2= j | y1=v) ), 
        % 4th column - j is closest value to v such that prob( j| y1=v) = max( prob(y2= j | y1=v) ) and j<=v
        % 5th column - j is closest value to v such that prob( j| y1=v) = max( prob(y2= j | y1=v) ) and j>=v

scale_ALL=[]; p_ALL=[]; covar_ALL=eye(nr_v*(nr_v-1));       
for j=1:nr_v;
    ind=find( y1 == v(j,1) & isnan(y2)==0 ); Ny=size(ind, 1); scale=sqrt(Ny); %1,sqrt(Ny),Ny
    
    if Ny>=2; % if size is positive, estimate conditional sample-probabilities (otherwise, they are missing)
    prob_v_condy1(:, j)=histc( y2(ind,1) , v')/Ny;    
    prob_vy1_max(j, 2)=prob_v_condy1(j,j); prob_vy1_max(j, 3)=max(prob_v_condy1(:,j));
    
    ind = find( prob_v_condy1(1:j,j) == prob_vy1_max(j, 3));
    if size(ind, 1)>=1; prob_vy1_max(j, 4)=v(ind(size(ind, 1),1), 1); 
    end
    ind = find( prob_v_condy1(j:nr_v,j) == prob_vy1_max(j, 3));
    if size(ind, 1)>=1; prob_vy1_max(j, 5)=v(j-1+ind(1,1), 1); 
    end
         
    p_other_guesses = NaN*zeros(nr_v-1,1);  % Now estimate multivariate p-value  
    covar_other_guesses = NaN.*zeros(nr_v-1, nr_v-1);            
    for i=1:nr_v;  % for all values that are not the agent's guess        
    if i~=j;    % Variance of each probability difference   Pr( y2=k | y1 = k)- Pr(y2=j | y1 = k) 
    if i>j; h=i-1; 
    else
        h=i;
    end
    p_other_guesses(h,1) = scale * ( prob_vy1_max(j, 2) - prob_v_condy1(i, j) );  % Mode difference  
    
    % %  % Variance-Covariance matrix of the test     
    covar_other_guesses(h,h) = prob_vy1_max(j, 2)*(1-prob_vy1_max(j, 2))+prob_v_condy1(i, j)*(1-prob_v_condy1(i, j)) ...
        +2*prob_vy1_max(j, 2)*prob_v_condy1(i, j); % Variance element    
    if covar_other_guesses(h,h)==0;   % This could happen if sample estimate of prob_v_condy1(i, j) or prob_v_condy1(j, j) are 0
    covar_other_guesses(h,h) = max( [2*prob_vy1_max(j, 2), 2*prob_v_condy1(i, j), ...
        prob_vy1_max(j, 2)*(1-prob_vy1_max(j, 2))+prob_v_condy1(i, j)*(1-prob_v_condy1(i, j)) +2*prob_vy1_max(j, 2)*prob_v_condy1(i, j)]);    
    end
    if covar_other_guesses(h,h) == 0; % This could happen if sample estimate of prob_v_condy1(i, j)=prob_v_condy1(j, j)=0
    covar_other_guesses(h,h) = 0.25;  % When this happens t = 0 and there is no effect of choosing a variance for the test (as long as it is >0)  
    end
    
    % Covariance elements  (for elements bigger than i)
    for i2=i+1:nr_v;      
    if  i2~=j; 
        if i2>j; h2=i2-1;
        else
            h2 = i2;
        end 
    covar_other_guesses(h2,h) = prob_vy1_max(j, 2)*(1-prob_vy1_max(j, 2)) + prob_vy1_max(j, 2)*(prob_v_condy1(i, j) + prob_v_condy1(i2, j))+...
        - prob_v_condy1(i, j)*prob_v_condy1(i2, j);
    if min([prob_v_condy1(i2, j), prob_v_condy1(i, j), prob_v_condy1(j, j)])==0 ;  % case where correlation is 1 
        covar_other_guesses(h2,h) = 0.95*covar_other_guesses(h2,h);    % (it can happen if prob_v_condy1(i, j) or prob_v_condy1(i2, j) is 0         
    end            % covar_other_guesses(h2,h) = prob_vy1_max(j, 2);  %covar_other_guesses(h2,h) = min(prob_v_condy1(i, j), prob_v_condy1(i2, j));      
        
    % Case where sample variance is missing, use covariance = 0 (although it makes no difference as long as it is a number with absolute value smaller than 1)
    if max( [2*prob_vy1_max(j, 2), 2*prob_v_condy1(i, j), ...
        prob_vy1_max(j, 2)*(1-prob_vy1_max(j, 2))+prob_v_condy1(i, j)*(1-prob_v_condy1(i, j)) +2*prob_vy1_max(j, 2)*prob_v_condy1(i, j)])==0;    
    covar_other_guesses(h2,h) = 0;   
    end
        
    covar_other_guesses(h,h2) = covar_other_guesses(h2,h);    % Put value for symmetric part of matrix  
    end
    end

    end
    end  
    
    %{     %}   
    scale_ALL=[scale_ALL;scale]; p_ALL=[p_ALL;p_other_guesses]; 
    a1=1+(j-1)*(nr_v-1); a2=j*(nr_v-1); covar_ALL(a1:a2,a1:a2)=covar_other_guesses;    
    end
end

p=nanmin(normcdf(p_ALL));     %x0 = zeros(1, size(p_ALL,1) ); % vector of initial guesses    %{     %}  
