function [p_joint, p_lower, p_upper, prob_v_condy1, prob_exp_quant]=DDS_quantile_test(y1, y2, alpha)
% Function gives the p-value of the test described by Das-Dominitz and Soest (1999) for the expectations of  discrete outcomes
% This function makes the test that agents are both rational and express expectations as their subjective Quantile Alpha

% y1 is a [N x 1] column vector of expectations
% y2 is a [N x 1] column vector of results 
% y1 and y2 are assumed to be discrete (i.e, values = 0, 1, 2, 3,..., n belong to integer numbers)  
% alpha is the quantile of the agents' expectations to be tested (note that alpha is a number between 0 and 1)

% Results:
% p_joint is the p-value for the joint test: Pr( y2<=k-1 | y1 = k)<= alpha and Pr( y2>=k+1 | y1 = k)<= 1-alpha
% p_lower is the p-value for the test: Pr( y2<=k-1 | y1 = k)< alpha
% p_upper is the p-value for the test: Pr( y2>=k+1 | y1 = k)<= 1-alpha
% prob_v_condy1 is the cdf  Pr( y2<= j | y1 = k) where j is row and k is column
% [prob_exp_quant has the values of the agents' guesses ordered in the first column (v),
% 2nd column has the sample estimates for the alpha quantile of the conditional cdf Pr( y2<=j | y1 = k)
% 3rd column has the sample estimate of the cdf Pr( y2<=k-1 | y1 = k)  - This allows to see how far the lower bound differs from <=0.5
% 4th column has the sample estimate of the cdf Pr( y2<=k | y1 = k) - This allows to see how far the upper bound differs from >=0.5
% 5th column has the sample estimate of the cdf Pr( y2<=quantile_y2(k) | y1 = k) for each value k

vmin = min([y1; y2]);  vmax = max([y1; y2]); v = vmin:vmax;
obs_v = histc([y1; y2], v);
for i = 1:size(obs_v, 1);
    if obs_v(i, 1)==0;
        v(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
v = v(logical( isnan(v)==0 ))';  % values that never appear will not be used for computing probabilities

nr_v = size(v, 1); prob_v_condy1 = NaN.*zeros(nr_v);
p_lower = NaN.*zeros(nr_v,1);  p_upper = NaN.*zeros(nr_v,1);
cov_upp_low = NaN.*zeros(nr_v,1);
prob_exp_quant = [v, NaN.*zeros(nr_v,4)]; % cdf of the agent's expectation and the true alpha quantile of y2

for j =1:nr_v;
    ind=find( y1 == v(j,1) & isnan(y2)==0 ); Ny=size(ind, 1); scale=sqrt(Ny); %1,sqrt(Ny),N
    
    if Ny>=1; % if size is positive, estimate conditional sample-probabilities (otherwise, they are missing)
    prob_v_condy1(:, j) = histc( y2(ind,1) , v')/Ny;
    
    if j>=2;
    p1 = sum(prob_v_condy1(1:j-1, j));
    p_lower(j,1) = (scale/sqrt( p1*(1-p1) )) * ( alpha -  p1 );  % Das-Dominitz-Soest test sqrt(Ny)
    else
    p_lower(j,1) = 1;  % If there is no lower category, we accept the test - classify it as 1       
    end
    
    if j<=nr_v-1;
    p2 = sum(prob_v_condy1(j+1:nr_v, j));        
    p_upper(j,1) = (scale/sqrt( p2*(1-p2) )) * ( (1-alpha) - p2 );  % Das-Dominitz-Soest test sqrt(Ny)
    else
    p_upper(j,1) = 1;   % If there is no upper category, we accept the test - classify it as 1  
    end
        
    prob_exp_quant(j, 2) = find( cumsum(prob_v_condy1(:, j)) >=alpha , 1, 'first');
    prob_exp_quant(j, 3) = sum(prob_v_condy1(1:j-1, j));    
    prob_exp_quant(j, 4) = sum(prob_v_condy1(1:j, j));
    prob_exp_quant(j, 5) = sum(prob_v_condy1(1:prob_exp_quant(j, 2), j));
    end
end

% now calculate p-value of one-sided confidence tests 
p_joint(1,1)  = normcdf( p_upper(1,1) );
p_joint(2:nr_v-1,1) = min(normcdf(p_lower(2:nr_v-1,1)), normcdf(p_upper(2:nr_v-1,1)) );
p_joint(nr_v,1)  = normcdf( p_lower(nr_v,1) );

p_lower(2:nr_v,1) = normcdf( p_lower(2:nr_v,1) );
p_upper(1:nr_v-1,1) = normcdf( p_upper(1:nr_v-1,1) );

prob_v_condy1  = cumsum( prob_v_condy1 );   % Cdf conditional probabilities
