function [p_joint]=DDS_quantile_test_ALL(y1, y2, alpha)
% Function gives the p-value of the test described by Madeira (2015) for the expectations of  discrete outcomes
% This function makes the test that agents are both rational and express expectations as their subjective Quantile Alpha
% It tests jointly that all the agents are rational for whatever prediction
% y1=1,...,K is reported

% y1 is a [N x 1] column vector of expectations
% y2 is a [N x 1] column vector of results 
% y1 and y2 are assumed to be discrete (i.e, values = 0, 1, 2, 3,..., n belong to integer numbers)  
% alpha is the quantile of the agents' expectations to be tested (note that alpha is a number between 0 and 1)

% Results:
% p_joint is the p-value for the joint test: Pr( y2<= k-1 | y1 = k)<= alpha and Pr( y2>= k+1 | y1 = k)<= 1-alpha
% p_lower is the p-value for the test: Pr( y2<= k-1 | y1 = k)<= alpha
% p_upper is the p-value for the test: Pr( y2>= k+1 | y1 = k)<= 1-alpha
% cov_upp_low is the estimated covariance between the tests p_lower and p_upper
% prob_v_condy1 is the cdf  Pr( y2<= j | y1 = k) where j is row and k is column

vmin=min([y1; y2]); vmax=max([y1; y2]); v=vmin:vmax; obs_v=histc([y1; y2], v);  %vmin=min([y1; y2]); vmax=max([y1; y2]); v=vmin:vmax; v=v';
for i = 1:size(obs_v, 1);
    if obs_v(i, 1)==0;
        v(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
v=v(logical( isnan(v)==0 ) )';  % values that never appear will not be used for computing probabilities

nr_v=size(v, 1);  prob_v_condy1=NaN.*zeros(nr_v);
p_lower=NaN.*zeros(nr_v,1); p_upper=p_lower; scale_lower=p_lower; scale_upper=p_lower;
cov_upp_low = NaN.*zeros(nr_v,1);

for j =1:nr_v;
    ind=find( y1 == v(j,1) & isnan(y2)==0 ); Ny=size(ind, 1); scale=sqrt(Ny); %1,sqrt(Ny)
    
    if Ny>=1; % if size is positive, estimate conditional sample-probabilities (otherwise, they are missing)
    prob_v_condy1(:, j) = histc( y2(ind,1) , v')/Ny; 
    
    if j>=2;
    p1=sum(prob_v_condy1(1:j-1, j)); p1_inv=1-p1;    
    var_p1=p1*(1-p1); %var_p1 = sum( prob_v_condy1(1:j-1, j).*repmat(p1_inv,j-1,1) ); %var_p1=p1*(1-p1);
    if var_p1<=0; var_p1=0.01; %No 0 variance allowed (gives infinite t-statistic)
    end
    p_lower(j,1) = (scale/sqrt(var_p1)) * ( alpha -  p1 );  % Das-Dominitz-Soest test
    scale_lower(j,1) = scale/sqrt(var_p1);
    else
    p_lower(j,1) = 1;  % If there is no lower category, we accept the test - classify it as 1       
    end
    
    if j<=nr_v-1;
    p2 = sum(prob_v_condy1(j+1:nr_v, j)); p2_inv=1-p2; 
    var_p2=p2*(1-p2); %var_p2 = sum( prob_v_condy1(j+1:nr_v, j).*repmat(p2_inv,nr_v-j,1) ); %var_p2=p2*(1-p2);
    if var_p2<=0; var_p2=0.01; %No 0 variance allowed (gives infinite t-statistic)
    end
    p_upper(j,1) = (scale/sqrt(var_p2)) * ( (1-alpha) - p2 );  % Das-Dominitz-Soest test
    scale_upper(j,1) = scale/sqrt(var_p2);    
    else
    p_upper(j,1) = 1;   % If there is no upper category, we accept the test - classify it as 1  
    end
    
    if j>=2 && j<=nr_v-1;
    cov_p12=-p1*p2; %cov_p12=-sum( prob_v_condy1(1:j-1, j).*repmat(p2,nr_v-j,1) ); %cov_p12=-p1*p2;  
    cov_upp_low(j,1) = cov_p12/sqrt(var_p1*var_p2); 
    else
    cov_upp_low(j,1) = 0;  % for values at the extreme of distribution, there is only one test, therefore there is no covariance   
    end
    
    end
end
 

a1=1+2*(nr_v-2)+1; p_ALL=zeros(a1,1); s_ALL=ones(a1,1); covar_ALL=eye(a1);  % % % % % %
p_ALL(1)=p_upper(1,1); s_ALL(1)=scale_upper(1);
warning('off','all');   % % % % % %
%p_joint(1,1)  = normcdf( p_upper(1,1) ); % now calculate p-value of one-sided confidence tests
%p_joint(nr_v,1)  = normcdf( p_lower(nr_v,1) );
for j=2:nr_v-1;  % Now use Kudo Test to evaluate the p-value of the joint test
x0=zeros(1, 2); % vector of initial guesses
p=[p_lower(j,1); p_upper(j,1)];  s=[scale_lower(j,1); scale_upper(j,1)];
covar=[1, cov_upp_low(j,1); cov_upp_low(j,1),  1];
a2=1+2*(j-2)+1; a3=a2+1; p_ALL(a2:a3)=p; s_ALL(a2:a3)=s; covar_ALL(a2:a3,a2:a3)=covar;
%{  %} 
end
warning('on','all');  % % % % % %

p_ALL(end)=p_lower(1,1); s_ALL(end)=scale_lower(end); warning('off','all');   % % % % % %
p_joint=nanmin(normcdf(p_ALL)); %x0=zeros(1, a1); %{  %}
warning('on','all');  % % % % % %

p_lower(2:nr_v,1) = normcdf( p_lower(2:nr_v,1) );
p_upper(1:nr_v-1,1) = normcdf( p_upper(1:nr_v-1,1) );
prob_v_condy1  = cumsum( prob_v_condy1 );   % Cdf conditional probabilities
