function [p_joint, p_lower, p_upper]=DDS_quantile_test_condX(y1, y2, x, alpha)
% Function gives the p-value of the test described by Das-Dominitz and Soest (1999) for the expectations of  discrete outcomes
% This function makes the test that agents are both rational and express expectations as their subjective Quantile Alpha
% This function differs from version 1 in the sense that it conditions results on the values of a x 

% y1 is a [N x 1] column vector of expectations
% y2 is a [N x 1] column vector of results 
% y1 and y2 are assumed to be discrete (i.e, values = 0, 1, 2, 3,..., n belong to integer numbers)  
% alpha is the quantile of the agents' expectations to be tested (note that alpha is a number between 0 and 1)
% x is the covariate that is used to condition the test of expectations (suggestions for x: race, sex or the value of the previous mark)
% I assume that x is a single covariate [N x 1]

% Results:
% p_joint is the p-value for the joint test: Pr( y2<= k-1 | y1 = k)<= alpha and Pr( y2>= k+1 | y1 = k)<= 1-alpha
% p_lower is the p-value for the test: Pr( y2<= k-1 | y1 = k)<= alpha
% p_upper is the p-value for the test: Pr( y2>= k+1 | y1 = k)<= 1-alpha
% cov_upp_low is the estimated covariance between the tests p_lower and p_upper
% prob_v_condy1 is the cdf  Pr( y2<= j | y1 = k) where j is row and k is column
% [prob_exp_quant has the values of the agents' guesses ordered in the first column (v),
% 2nd column has the sample estimates for the alpha quantile of the conditional cdf Pr( y2<=j | y1 = k)
% 3rd column has the sample estimate of the cdf Pr( y2<=k-1 | y1 = k)  - This allows to see how far the lower bound differs from <=0.5
% 4th column has the sample estimate of the cdf Pr( y2<=k | y1 = k) - This allows to see how far the upper bound differs from >=0.5
% 5th column has the sample estimate of the cdf Pr( y2<=quantile_y2(k) | y1 = k) for each value k

vmin = min([y1; y2]);
vmax = max([y1; y2]);

v = vmin:vmax;
obs_v = histc([y1; y2], v);
for i = 1:size(obs_v, 1);
    if obs_v(i, 1)==0;
        v(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
v = v(logical( isnan(v)==0 ) )';  % values that never appear will not be used for computing probabilities

nr_v = size(v, 1);


xmin = min(x);
xmax = max(x);
xv = xmin:xmax;
obs_x = histc(x, xv);
for i = 1:size(obs_x, 1);
    if obs_x(i, 1)==0;
        xv(1, i) =NaN;  % if value has no obs, mark it as missing
    end
end
xv = xv(logical( isnan(xv)==0 ) )';  % values that never appear will not be used for computing probabilities
nr_x = size(xv, 1);

p_joint = NaN.*zeros(nr_v, nr_x);
p_lower = NaN.*zeros(nr_v, nr_x); 
p_upper = NaN.*zeros(nr_v, nr_x); 
cov_upp_low = NaN.*zeros(nr_v, nr_x);

for i =1:nr_x;
    ind = logical(x==xv(i,1));
    [p_joint(:, i), p_lower(:, i), p_upper(:, i)] =  DDS_quantile_test(y1(ind), y2(ind), alpha); %das_dominitz_soest_quantile_test(y1(ind), y2(ind), alpha);
end
