% This file performs Monte Carlo tests suggested by Das, Dominitz and Soest (1999) and Madeira (2015) for expectations of categorical variables
% We test therefore 1) the joint hypothesis that agents are rational and express their subjective Mode
% 2) the joint hypothesis that agents are rational and express their subjective Median

clear all; close all; tic;

% 0 - Format of the output
clevel=0.05; % confidence level of test for not-rejecting rationality
values=[1,2,3,4]'; nr_v=length(values); % Categorical outcomes and their nr
MC_nr=10000; % Nr of Monte Carlo replicas

% 1st column - Prob of Outcomes 1,2,3,4 conditional on Expectation being 1 % (group does not reject either Mode - Mode binds for some restrictions)
% 2nd column - Prob of Outcomes 1,2,3,4 conditional on Expectation being 2 % (group does not reject either Mode or Median - Median binds, Mode binds for some)
% 3rd column - Prob of Outcomes 1,2,3,4 conditional on Expectation being 3 % (group does reject Median or Mode rationality - Mode binds for some restrictions)
% 4th column - Prob of Outcomes 1,2,3,4 conditional on Expectation being 4 % (group violates Median rationality but not Mode - Mode binds)

Prob_condExp = [[0.35, 0.35, 0.15, 0.15]', [0.15, 0.35, 0.35, 0.15]', [0.10, 0.20, 0.35, 0.35]', [0.25, 0.25, 0.25, 0.25]']; %Prob_condExp = [[0.55, 0.25, 0.15, 0.05]', [0.20, 0.50, 0.20, 0.10]', [0.10, 0.35, 0.30, 0.25]', [0.05, 0.10, 0.50, 0.35]'];
Cdf_condExp = cumsum(Prob_condExp); % Cdf of Outcomes
Cdf1_condExp=zeros(nr_v); Cdf1_condExp(2:end,:)=Cdf_condExp(1:end-1,:); % Cdf of Outcome k-1
Mode_assumption=[1,1,1,1]'; Median_assumption=[0,1,1,0]';

Prob_Exp=[0.25, 0.25, 0.25, 0.25]'; % Each group of Forecasters has the same probability in the population
Sample_eachExp=[25, 50, 75, 100, 150, 250, 500, 1000, 2500, 10000]';  % Size of groups for performing test statistic

Type1_error_Mode_DDS=zeros(nr_v, length(Sample_eachExp)); Type2_error_Mode_DDS=Type1_error_Mode_DDS;
Type1_error_Mode_MultiC=Type1_error_Mode_DDS;  Type2_error_Mode_MultiC=Type1_error_Mode_DDS;
Type1_error_Median_DDS=Type1_error_Mode_DDS; Type2_error_Median_DDS=Type1_error_Mode_DDS;
Type1_error_Median_MultiC=Type1_error_Mode_DDS;  Type2_error_Median_MultiC=Type1_error_Mode_DDS;

for MCsize=1:length(Sample_eachExp); N=Sample_eachExp(MCsize);
    ind=ones(N,1); Exp=[1*ind;2*ind;3*ind;4*ind];
    Test_Mode_DDS=NaN.*zeros(nr_v,MC_nr); Test_Mode_MultiC=Test_Mode_DDS;
    Test_Median_DDS=Test_Mode_DDS; Test_Median_MultiC=Test_Mode_DDS;
    
    for MC_rep=1:MC_nr; Y=zeros(N,nr_v);
        for out1=1:nr_v; Uy=rand(N,1); % Create Outcomes conditional on Expectations
        for out2=1:nr_v; ind=logical(Uy>Cdf1_condExp(out2,out1) & Uy<=Cdf_condExp(out2,out1));  Y(ind,out1)=out2;        
        end
        end
        Y=reshape(Y,numel(Y),1); % Remember HR=1 is accepting rationality
        p=DDS_mode_test(Exp,Y); HR=logical(p>=clevel); Test_Mode_DDS(:,MC_rep)=HR; % DDS (1999) Mode test
        p=Multiplecondition_mode_test(Exp,Y); HR=logical(p>=clevel); Test_Mode_MultiC(:,MC_rep)=HR; % Madeira (2015) Mode test  
        p=DDS_quantile_test(Exp,Y,0.50); HR=logical(p>=clevel); Test_Median_DDS(:,MC_rep)=HR; % DDS (1999) Median test
        p=Multiplecondition_quantile_test(Exp,Y,0.50); HR=logical(p>=clevel); Test_Median_MultiC(:,MC_rep)=HR; % Madeira (2015) Median test
    end
    for out1=1:nr_v;  % Summarize Type I and Type II errors
    if Mode_assumption(out1)==1;  %Type I error only applies if Rationality is true
    ind=logical(Test_Mode_DDS(out1,:)~=1); Type1_error_Mode_DDS(out1,MCsize)=sum(ind); % Remember HR=1 is accepting rationality
    ind=logical(Test_Mode_MultiC(out1,:)~=1); Type1_error_Mode_MultiC(out1,MCsize)=sum(ind);
    else   %Type II error only applies if Rationality is false
    ind=logical(Test_Mode_DDS(out1,:)~=0); Type2_error_Mode_DDS(out1,MCsize)=sum(ind);
    ind=logical(Test_Mode_MultiC(out1,:)~=0); Type2_error_Mode_MultiC(out1,MCsize)=sum(ind);        
    end
    if Median_assumption(out1)==1;  %Type I error only applies if Rationality is true
    ind=logical(Test_Median_DDS(out1,:)~=1); Type1_error_Median_DDS(out1,MCsize)=sum(ind);
    ind=logical(Test_Median_MultiC(out1,:)~=1); Type1_error_Median_MultiC(out1,MCsize)=sum(ind);
    else   %Type II error only applies if Rationality is false
    ind=logical(Test_Median_DDS(out1,:)~=0); Type2_error_Median_DDS(out1,MCsize)=sum(ind);
    ind=logical(Test_Median_MultiC(out1,:)~=0); Type2_error_Median_MultiC(out1,MCsize)=sum(ind);        
    end
    
    end
       
end
[prob_v_cond] = Prob_outcomes_condY1(Exp, reshape(Y,numel(Y),1)); % Probability of Outcomes given Expectations

Type1_error_Mode_DDS=Type1_error_Mode_DDS/MC_nr; Type2_error_Mode_DDS=Type2_error_Mode_DDS/MC_nr;
Type1_error_Mode_MultiC=Type1_error_Mode_MultiC/MC_nr;  Type2_error_Mode_MultiC=Type2_error_Mode_MultiC/MC_nr;
Type1_error_Median_DDS=Type1_error_Median_DDS/MC_nr; Type2_error_Median_DDS=Type2_error_Median_DDS/MC_nr;
Type1_error_Median_MultiC=Type1_error_Median_MultiC/MC_nr; Type2_error_Median_MultiC=Type2_error_Median_MultiC/MC_nr;

% 3 - Saving results
save('MonteCarlo_DDS_tests_5'); toc; % End of program
